@@ -1,30 +1,21 @@
-import warnings
-from django.contrib.sites.models import get_current_site
-from django.core.exceptions import ImproperlyConfigured
-from django.utils.encoding import smart_str
-from fluent_comments import appsettings
-from .compat import BASE_APP
+from __future__ import absolute_import
 
-if BASE_APP == 'django.contrib.comments':
-    from django.contrib.comments.moderation import moderator, CommentModerator
-elif BASE_APP == 'django_comments':
-    from django_comments.moderation import moderator, CommentModerator
-else:
-    raise NotImplementedError()
+import logging
+
+from akismet import SpamStatus
+from django_comments.moderation import moderator, CommentModerator
+
+from fluent_comments import appsettings
+from fluent_comments.akismet import akismet_check
+from fluent_comments.email import send_comment_posted
+from fluent_comments.utils import split_words
 
 try:
     from urllib.parse import urljoin  # Python 3
 except ImportError:
     from urlparse import urljoin  # Python 2
 
-# Optional dependency (for lacking Python 3 support)
-try:
-    from akismet import Akismet
-except ImportError:
-    Akismet = None
-    if appsettings.FLUENT_CONTENTS_USE_AKISMET:
-        warnings.warn("No `akismet` package has been installed, disabling Akismet checks for django-fluent-comments.", RuntimeWarning)
-
+logger = logging.getLogger(__name__)
 
 # Akismet code originally based on django-comments-spamfighter.
 
@@ -47,10 +38,10 @@ class FluentCommentsModerator(CommentModerator):
 
     close_after = appsettings.FLUENT_COMMENTS_CLOSE_AFTER_DAYS
     moderate_after = appsettings.FLUENT_COMMENTS_MODERATE_AFTER_DAYS
-    email_notification = False   # Using signals instead
-    akismet_check = appsettings.FLUENT_CONTENTS_USE_AKISMET and Akismet is not None
+    email_notification = appsettings.FLUENT_COMMENTS_USE_EMAIL_NOTIFICATION
+    akismet_check = appsettings.FLUENT_CONTENTS_USE_AKISMET
     akismet_check_action = appsettings.FLUENT_COMMENTS_AKISMET_ACTION
-
+    moderate_bad_words = set(appsettings.FLUENT_COMMENTS_MODERATE_BAD_WORDS)
 
     def allow(self, comment, content_object, request):
         """
@@ -63,13 +54,15 @@ class FluentCommentsModerator(CommentModerator):
             return False
 
         # Akismet check
-        if self.akismet_check and self.akismet_check_action == 'delete':
-            if self._akismet_check(comment, content_object, request):
+        if self.akismet_check:
+            akismet_result = akismet_check(comment, content_object, request)
+            if self.akismet_check_action == 'delete' and akismet_result in (SpamStatus.ProbableSpam, SpamStatus.DefiniteSpam):
                 return False  # Akismet marked the comment as spam.
+            elif self.akismet_check_action == 'auto' and akismet_result == SpamStatus.DefiniteSpam:
+                return False  # Clearly spam
 
         return True
 
-
     def moderate(self, comment, content_object, request):
         """
         Determine whether a given comment on a given object should be allowed to show up immediately,
@@ -77,78 +70,86 @@ class FluentCommentsModerator(CommentModerator):
 
         Returns ``True`` if the comment should be moderated (marked non-public), ``False`` otherwise.
         """
+
+        # Soft delete checks are done first, so these comments are not mistakenly "just moderated"
+        # for expiring the `close_after` date, but correctly get marked as spam instead.
+        # This helps staff to quickly see which comments need real moderation.
+        if self.akismet_check:
+            akismet_result = akismet_check(comment, content_object, request)
+            if akismet_result:
+                # Typically action=delete never gets here, unless the service was having problems.
+                if akismet_result in (SpamStatus.ProbableSpam, SpamStatus.DefiniteSpam) and \
+                       self.akismet_check_action in ('auto', 'soft_delete', 'delete'):
+                   comment.is_removed = True  # Set extra marker
+
+                # SpamStatus.Unknown or action=moderate will end up in the moderation queue
+                return True
+
         # Parent class check
         if super(FluentCommentsModerator, self).moderate(comment, content_object, request):
             return True
 
+        # Bad words check
+        if self.moderate_bad_words:
+            input_words = split_words(comment.comment)
+            if self.moderate_bad_words.intersection(input_words):
+                return True
+
         # Akismet check
-        if self.akismet_check and self.akismet_check_action == 'moderate':
+        if self.akismet_check and self.akismet_check_action not in ('soft_delete', 'delete'):
             # Return True if akismet marks this comment as spam and we want to moderate it.
-            if self._akismet_check(comment, content_object, request):
+            if akismet_check(comment, content_object, request):
                 return True
 
         return False
 
-
-    def _akismet_check(self, comment, content_object, request):
+    def email(self, comment, content_object, request):
         """
-        Connects to Akismet and returns True if Akismet marks this comment as
-        spam. Otherwise returns False.
+        Overwritten for a better email notification.
         """
-        # Get Akismet data
-        AKISMET_API_KEY = appsettings.AKISMET_API_KEY
-        if not AKISMET_API_KEY:
-            raise ImproperlyConfigured('You must set AKISMET_API_KEY to use comment moderation with Akismet.')
+        if not self.email_notification:
+            return
 
-        current_domain = get_current_site(request).domain
-        auto_blog_url = '{0}://{1}/'.format(request.is_secure() and 'https' or 'http', current_domain)
-        blog_url = appsettings.AKISMET_BLOG_URL or auto_blog_url
+        send_comment_posted(comment, request)
 
-        akismet_api = Akismet(
-            key=AKISMET_API_KEY,
-            blog_url=blog_url
+
+class NullModerator(FluentCommentsModerator):
+    """
+    A moderator class that has the same effect as not being here.
+    It allows all comments, disabling all moderation.
+    This can be used in ``FLUENT_COMMENTS_DEFAULT_MODERATOR``.
+    """
+
+    def allow(self, comment, content_object, request):
+        return True
+
+    def moderate(self, comment, content_object, request):
+        logger.info("Unconditionally allow comment, no default moderation set.")
+        return False
+
+
+class AlwaysModerate(FluentCommentsModerator):
+    """
+    A moderator class that will always mark the comment as moderated.
+    This can be used in ``FLUENT_COMMENTS_DEFAULT_MODERATOR``.
+    """
+    def moderate(self, comment, content_object, request):
+        # Still calling super in case Akismet marks the comment as spam.
+        return super(AlwaysModerate, self).moderate(comment, content_object, request) or True
+
+
+class AlwaysDeny(FluentCommentsModerator):
+    """
+    A moderator that will deny any comments to be posted.
+    This can be used in ``FLUENT_COMMENTS_DEFAULT_MODERATOR``.
+    """
+    def allow(self, comment, content_object, request):
+        logger.warning(
+            "Discarded comment on unregistered model '%s'",
+            content_object.__class__.__name__
         )
-
-        if akismet_api.verify_key():
-            akismet_data = self._get_akismet_data(blog_url, comment, content_object, request)
-            if akismet_api.comment_check(smart_str(comment.comment), data=akismet_data, build_data=True):
-                return True
-
         return False
 
-    def _get_akismet_data(self, blog_url, comment, content_object, request):
-        # Field documentation:
-        # http://akismet.com/development/api/#comment-check
-        akismet_data = {
-            # Comment info
-            'permalink': urljoin(blog_url, content_object.get_absolute_url()),
-            'comment_type': 'comment',   # comment, trackback, pingback, see http://blog.akismet.com/2012/06/19/pro-tip-tell-us-your-comment_type/
-            'comment_author': getattr(comment, 'name', ''),
-            'comment_author_email': getattr(comment, 'email', ''),
-            'comment_author_url': getattr(comment, 'url', ''),
-
-            # Request info
-            'referrer': request.META.get('HTTP_REFERER', ''),
-            'user_agent': request.META.get('HTTP_USER_AGENT', ''),
-            'user_ip': comment.ip_address,
-
-            # Server info
-            'SERVER_ADDR': request.META.get('SERVER_ADDR', ''),
-            'SERVER_ADMIN': request.META.get('SERVER_ADMIN', ''),
-            'SERVER_NAME': request.META.get('SERVER_NAME', ''),
-            'SERVER_PORT': request.META.get('SERVER_PORT', ''),
-            'SERVER_SIGNATURE': request.META.get('SERVER_SIGNATURE', ''),
-            'SERVER_SOFTWARE': request.META.get('SERVER_SOFTWARE', ''),
-            'HTTP_ACCEPT': request.META.get('HTTP_ACCEPT', ''),
-        }
-
-        # Allow testing, see:
-        # http://blog.akismet.com/2012/07/20/pro-tip-testing-testing/
-        if appsettings.AKISMET_IS_TEST:
-            akismet_data['is_test'] = '1'
-
-        return akismet_data
-
 
 def moderate_model(ParentModel, publication_date_field=None, enable_comments_field=None):
     """
