@@ -1,15 +1,20 @@
+import json
+import sys
+
+import django_comments
+from django.apps import apps
 from django.core.exceptions import ObjectDoesNotExist, ValidationError
-from django.db import models
 from django.http import HttpResponse, HttpResponseBadRequest
 from django.template.loader import render_to_string
-from django.template import RequestContext
 from django.utils.html import escape
 from django.views.decorators.csrf import csrf_protect
 from django.views.decorators.http import require_POST
-from .compat import get_form as get_comments_form, signals, CommentPostBadRequest
+from django_comments import signals
+from django_comments.views.comments import CommentPostBadRequest
+
+from fluent_comments.compat import is_authenticated
+from fluent_comments.utils import get_comment_template_name, get_comment_context_data
 from fluent_comments import appsettings
-import json
-import sys
 
 if sys.version_info[0] >= 3:
     long = int
@@ -31,10 +36,9 @@ def post_comment_ajax(request, using=None):
     # This is a separate view to integrate both features. Previously this used django-ajaxcomments
     # which is unfortunately not thread-safe (it it changes the comment view per request).
 
-
     # Fill out some initial data fields from an authenticated user, if present
     data = request.POST.copy()
-    if request.user.is_authenticated():
+    if is_authenticated(request.user):
         if not data.get('name', ''):
             data["name"] = request.user.get_full_name() or request.user.username
         if not data.get('email', ''):
@@ -46,12 +50,11 @@ def post_comment_ajax(request, using=None):
     if ctype is None or object_pk is None:
         return CommentPostBadRequest("Missing content_type or object_pk field.")
     try:
-        object_pk = long(object_pk)
-        model = models.get_model(*ctype.split(".", 1))
+        model = apps.get_model(*ctype.split(".", 1))
         target = model._default_manager.using(using).get(pk=object_pk)
     except ValueError:
         return CommentPostBadRequest("Invalid object_pk value: {0}".format(escape(object_pk)))
-    except TypeError:
+    except (TypeError, LookupError):
         return CommentPostBadRequest("Invalid content_type value: {0}".format(escape(ctype)))
     except AttributeError:
         return CommentPostBadRequest("The given content-type {0} does not resolve to a valid model.".format(escape(ctype)))
@@ -61,34 +64,33 @@ def post_comment_ajax(request, using=None):
         return CommentPostBadRequest("Attempting go get content-type {0!r} and object PK {1!r} exists raised {2}".format(escape(ctype), escape(object_pk), e.__class__.__name__))
 
     # Do we want to preview the comment?
-    preview = "preview" in data
+    is_preview = "preview" in data
 
     # Construct the comment form
-    form = get_comments_form()(target, data=data)
+    form = django_comments.get_form()(target, data=data, is_preview=is_preview)
 
     # Check security information
     if form.security_errors():
         return CommentPostBadRequest("The comment form failed security verification: {0}".format(form.security_errors()))
 
     # If there are errors or if we requested a preview show the comment
-    if preview:
+    if is_preview:
         comment = form.get_comment_object() if not form.errors else None
         return _ajax_result(request, form, "preview", comment, object_id=object_pk)
     if form.errors:
         return _ajax_result(request, form, "post", object_id=object_pk)
 
-
     # Otherwise create the comment
     comment = form.get_comment_object()
     comment.ip_address = request.META.get("REMOTE_ADDR", None)
-    if request.user.is_authenticated():
+    if is_authenticated(request.user):
         comment.user = request.user
 
     # Signal that the comment is about to be saved
     responses = signals.comment_will_be_posted.send(
-        sender  = comment.__class__,
-        comment = comment,
-        request = request
+        sender=comment.__class__,
+        comment=comment,
+        request=request
     )
 
     for (receiver, response) in responses:
@@ -131,14 +133,12 @@ def _ajax_result(request, form, action, comment=None, object_id=None):
     }
 
     if comment is not None:
-        context = {
-            'comment': comment,
-            'action': action,
-            'preview': (action == 'preview'),
-            'USE_THREADEDCOMMENTS': appsettings.USE_THREADEDCOMMENTS,
-        }
-        comment_html = render_to_string('comments/comment.html', context, context_instance=RequestContext(request))
+        # Render the comment, like {% render_comment comment %} does
+        context = get_comment_context_data(comment, action)
+        context['request'] = request
+        template_name = get_comment_template_name(comment)
 
+        comment_html = render_to_string(template_name, context, request=request)
         json_return.update({
             'html': comment_html,
             'comment_id': comment.id,
